# frozen_string_literal: true

require 'enqueue_export/version'

module EnqueueExport
  class Error < StandardError; end
  attr_accessor :pending_exports

  def self.perform(model)
    group_by(model).each do |data|
      batch_size(data).each do |batch|
        worker_id = generate_id
        ids = batch.map(&:id)
        public_send(model).where(id: ids).update_all(worker_id: worker_id)
        delayed_job(model, worker_id)
      end
    end
  end

  def self.generate_id
    SecureRandom.hex(32)
  end

  def self.export
    @pending_export = PendingExport.where(local_feed_id: nil, worker_id: nil)
    @pending_export.where(response_id: nil, processed: false)
  end

  def self.response
    @pending_export = PendingExport.where(local_feed_id: nil, worker_id: nil)
    @pending_export.where(processed: false).where.not(response_id: nil)
  end

  def self.validity
    @pending_validity = PendingValidity.where(
      worker_id: nil,
      validity: false,
      processed: false
    )
  end

  def self.qc
    @pending_qc = PendingQc.where(worker_id: nil)
  end

  def self.group_by(model)
    public_send(model).group_by do |obj|
      [obj.mode, obj.account_id, obj.channel_id]
    end.to_a
  end

  def self.batch_size(data)
    args = {
      channel_id: data[0][2],
      name: 'batch_size'
    }
    size = Config.find_by(args).value.to_i
    data[1].in_groups_of(size, false)
  end

  def self.delayed_job(model, worker_id)
    service = if %w[export response].include? model
                Kernel.const_get("Worker::#{model.capitalize}Service")
              else
                Kernel.const_get("Check#{model.capitalize}Service")
              end
    Delayed::Job.enqueue(service.new(worker_id: worker_id))
  end
end
